const fs = require('fs')
const convert = require('xml-js')

const Minimatch = require("minimatch").Minimatch

const flatten = (node, parent) =>
    [].concat(
        {node, parent},
        ...(node.elements || []).map(e => flatten(e, node))
    )

const postprocess = str => str.replace(/([^\s])\/>/g, "$1 />")

const getIncludeAttribute = n => n.node.attributes.Include.replace(/\\/g, "/")

const update = (projfile, globs, files) => {
    const matchers = globs.map(pattern => new Minimatch(pattern))
    const fileMatchers = files.map(file => new Minimatch(file))
    fs.readFile(projfile, function(err, data) {
        const csproj = convert.xml2js(data)
        const elements = flatten(csproj)
        const contents = elements
            .filter(n => n.node.name == "Content")
        const contentsMatchedByGlobs = contents
            .filter(n => matchers.some(m => m.match(getIncludeAttribute(n))))
        const contentsToDelete = contentsMatchedByGlobs
            .filter(n => !fileMatchers.some(m => m.match(getIncludeAttribute(n))))
        contentsToDelete
            .forEach(n => n.parent.elements = n.parent.elements.filter(node => node !== n.node))

        const filesToAdd = files
            .filter(file => !contents.some(n => getIncludeAttribute(n) == file))
        if (filesToAdd.length > 0){
            const contentsToAdd = filesToAdd.map(file => ({
                type: "element",
                name: "Content",
                attributes: {
                    Include: file.replace(/\//g, "\\")
                }
            }))
            const elements = csproj.elements[0].elements
            const lastElement = elements[elements.length - 1]

            if (lastElement.name == "ItemGroup"){
                lastElement.elements = [].concat(
                    lastElement.elements,
                    contentsToAdd
                ).sort( (a, b) => {
                    if (!a.attributes.Include || !b.attributes.Include){
                        return -1
                    }
                    if (a.attributes.Include < b.attributes.Include){
                        return -1
                    }
                    if (a.attributes.Include > b.attributes.Include){
                        return 1
                    }
                    return 0
                })
            } else {
                csproj.elements[0].elements.push({
                    type: "element",
                    name: "ItemGroup",
                    elements: contentsToAdd
                })
            }
        }

        const output = postprocess(convert.js2xml(csproj, {spaces: 2}))
        fs.writeFile(projfile, output, err => err && console.error(err))
    });
}

module.exports = {
    update
}